@@ -6,6 +6,7 @@
     email                : mte@users.sourceforge.net
                              Javier Campos Morales <javi_c@ctv.es>
                              Felix Rodriguez <frodriguez@mail.wesleyan.edu>
+                             (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -22,56 +23,62 @@
 // ----------------------------------------------------------------------------
 // QT Includes
 
-#include <QPixmap>
-#include <QLabel>
 #include <QCheckBox>
+#include <QUrl>
+#include <QPushButton>
+#include <QIcon>
+#include <QFileDialog>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <kglobal.h>
-#include <klocale.h>
-#include <kstandarddirs.h>
-#include <kconfig.h>
-#include <kglobalsettings.h>
-#include <kpushbutton.h>
-#include <kiconloader.h>
-#include <kguiitem.h>
-#include <kfiledialog.h>
+#include <KConfig>
+#include <KConfigGroup>
+#include <KLocalizedString>
+#include <KSharedConfig>
 
 // ----------------------------------------------------------------------------
 // Project Includes
 
-KBackupDlg::KBackupDlg(QWidget* parent)
-    : kbackupdlgdecl(parent)
+#include "ui_kbackupdlg.h"
+
+#include "icons/icons.h"
+
+using namespace Icons;
+
+KBackupDlg::KBackupDlg(QWidget* parent) :
+  QDialog(parent),
+  ui(new Ui::KBackupDlg)
 {
+  ui->setupUi(this);
   readConfig();
 
-  // add icons to buttons
-  btnOK->setGuiItem(KStandardGuiItem::ok());
-  btnCancel->setGuiItem(KStandardGuiItem::cancel());
+  ui->chooseButton->setIcon(Icons::get(Icon::Folder));
 
-  KGuiItem chooseButtenItem(i18n("C&hoose..."),
-                            KIcon("folder"),
-                            i18n("Select mount point"),
-                            i18n("Use this to browse to the mount point."));
-  chooseButton->setGuiItem(chooseButtenItem);
-
-  connect(chooseButton, SIGNAL(clicked()), this, SLOT(chooseButtonClicked()));
-  connect(btnOK, SIGNAL(clicked()), this, SLOT(accept()));
-  connect(btnCancel, SIGNAL(clicked()), this, SLOT(reject()));
+  connect(ui->chooseButton, &QAbstractButton::clicked, this, &KBackupDlg::chooseButtonClicked);
 }
 
 KBackupDlg::~KBackupDlg()
 {
   writeConfig();
+  delete ui;
+}
+
+QString KBackupDlg::mountPoint() const
+{
+  return ui->txtMountPoint->text();
+}
+
+bool KBackupDlg::mountCheckBox() const
+{
+  return ui->mountCheckBox;
 }
 
 void KBackupDlg::chooseButtonClicked()
 {
-  KUrl newDir = KFileDialog::getExistingDirectoryUrl(KUrl::fromPath(KGlobalSettings::documentPath()));
-  if (newDir.hasPath())
-    txtMountPoint->setText(newDir.path());
+  auto newDir = QFileDialog::getExistingDirectoryUrl(this, QString(), QUrl::fromLocalFile(QStandardPaths::writableLocation(QStandardPaths::DocumentsLocation)));
+  if (!newDir.path().isEmpty())
+    ui->txtMountPoint->setText(newDir.path());
 }
 
 void KBackupDlg::readConfig()
@@ -82,17 +89,17 @@ void KBackupDlg::readConfig()
 #else
   backupDefaultLocation = "/mnt/floppy";
 #endif
-  KSharedConfigPtr config = KGlobal::config();
+  KSharedConfigPtr config = KSharedConfig::openConfig();
   KConfigGroup grp = config->group("Last Use Settings");
-  mountCheckBox->setChecked(grp.readEntry("KBackupDlg_mountDevice", false));
-  txtMountPoint->setText(grp.readEntry("KBackupDlg_BackupMountPoint", backupDefaultLocation));
+  ui->mountCheckBox->setChecked(grp.readEntry("KBackupDlg_mountDevice", false));
+  ui->txtMountPoint->setText(grp.readEntry("KBackupDlg_BackupMountPoint", backupDefaultLocation));
 }
 
 void KBackupDlg::writeConfig()
 {
-  KSharedConfigPtr config = KGlobal::config();
+  KSharedConfigPtr config = KSharedConfig::openConfig();
   KConfigGroup grp = config->group("Last Use Settings");
-  grp.writeEntry("KBackupDlg_mountDevice", mountCheckBox->isChecked());
-  grp.writeEntry("KBackupDlg_BackupMountPoint", txtMountPoint->text());
+  grp.writeEntry("KBackupDlg_mountDevice", ui->mountCheckBox->isChecked());
+  grp.writeEntry("KBackupDlg_BackupMountPoint", ui->txtMountPoint->text());
   config->sync();
 }
