@@ -4,6 +4,7 @@
     begin                : Tue Feb 5 2008
     copyright            : (C) 2008 by Thomas Baumgart
     email                : Thomas Baumgart <ipwizard@users.sourceforge.net>
+                           (C) 2017 by Łukasz Wojniłowicz <lukasz.wojnilowicz@gmail.com>
  ***************************************************************************/
 
 /***************************************************************************
@@ -15,8 +16,6 @@
  *                                                                         *
  ***************************************************************************/
 
-#include <config-kmymoney.h>
-
 #include "kaccounttemplateselector.h"
 
 // ----------------------------------------------------------------------------
@@ -26,46 +25,38 @@
 #include <QTimer>
 #include <QList>
 #include <QTreeWidget>
+#include <QStandardPaths>
 
 // ----------------------------------------------------------------------------
 // KDE Includes
 
-#include <kglobal.h>
-#include <klocale.h>
-#include <kstandarddirs.h>
-#include <ktextedit.h>
+#include <KTextEdit>
 
 // ----------------------------------------------------------------------------
 // Project Includes
+
+#include "ui_kaccounttemplateselector.h"
+
 #include <mymoneytemplate.h>
 
-class KAccountTemplateSelector::Private
+class KAccountTemplateSelectorPrivate
 {
+  Q_DISABLE_COPY(KAccountTemplateSelectorPrivate)
+
 public:
-  Private(KAccountTemplateSelector* p) {
-    m_parent = p;
+  KAccountTemplateSelectorPrivate() :
+    ui(new Ui::KAccountTemplateSelector),
+    id(0)
+  {
   }
-#ifndef KMM_DESIGNER
-  QList<MyMoneyTemplate> selectedTemplates() const;
-  QTreeWidgetItem* hierarchyItem(const QString& parent, const QString& name);
-  void loadHierarchy();
-#endif
-
-public:
-  KAccountTemplateSelector*        m_parent;
-  QMap<QString, QTreeWidgetItem*>  m_templateHierarchy;
-#ifndef KMM_DESIGNER
-  QMap<QString, MyMoneyTemplate>   m_templates;
-  QMap<QString, QString>           countries;
-  QMap<QString, QString>::iterator it_m;
-  QStringList                      dirlist;
-  int                              id;
-#endif
-};
 
+  ~KAccountTemplateSelectorPrivate()
+  {
+    delete ui;
+  }
 
 #ifndef KMM_DESIGNER
-QTreeWidgetItem* KAccountTemplateSelector::Private::hierarchyItem(const QString& parent, const QString& name)
+QTreeWidgetItem* hierarchyItem(const QString& parent, const QString& name)
 {
   if (!m_templateHierarchy.contains(parent)
       || m_templateHierarchy[parent] == 0) {
@@ -78,13 +69,13 @@ QTreeWidgetItem* KAccountTemplateSelector::Private::hierarchyItem(const QString&
   return item;
 }
 
-void KAccountTemplateSelector::Private::loadHierarchy()
+void loadHierarchy()
 {
   m_templateHierarchy.clear();
-  QTreeWidgetItemIterator it(m_parent->m_groupList, QTreeWidgetItemIterator::Selected);
+  QTreeWidgetItemIterator it(ui->m_groupList, QTreeWidgetItemIterator::Selected);
   QTreeWidgetItem* it_v;
   while ((it_v = *it) != 0) {
-    m_templates[it_v->data(0, IdRole).toString()].hierarchy(m_templateHierarchy);
+    m_templates[it_v->data(0, Qt::UserRole).toInt()].hierarchy(m_templateHierarchy);
     ++it;
   }
 
@@ -96,7 +87,7 @@ void KAccountTemplateSelector::Private::loadHierarchy()
   // add the hierarchy from the MyMoneyFile object
   QList<MyMoneyAccount> aList;
   QList<MyMoneyAccount>::const_iterator it_a;
-  MyMoneyFile* file = MyMoneyFile::instance();
+  auto file = MyMoneyFile::instance();
   file->accountList(aList);
   if (aList.count() > 0) {
     m_templateHierarchy[file->accountToCategory(file->asset().id(), true)] = 0;
@@ -111,13 +102,12 @@ void KAccountTemplateSelector::Private::loadHierarchy()
   }
 #endif
 
-  m_parent->m_accountList->clear();
-  QMap<QString, QTreeWidgetItem*>::iterator it_m;
+  ui->m_accountList->clear();
 
   QRegExp exp("(.*):(.*)");
-  for (it_m = m_templateHierarchy.begin(); it_m != m_templateHierarchy.end(); ++it_m) {
+  for (QMap<QString, QTreeWidgetItem*>::iterator it_m = m_templateHierarchy.begin(); it_m != m_templateHierarchy.end(); ++it_m) {
     if (exp.indexIn(it_m.key()) == -1) {
-      (*it_m) = new QTreeWidgetItem(m_parent->m_accountList);
+      (*it_m) = new QTreeWidgetItem(ui->m_accountList);
       (*it_m)->setText(0, it_m.key());
     } else {
       (*it_m) = hierarchyItem(exp.cap(1), exp.cap(2));
@@ -125,19 +115,19 @@ void KAccountTemplateSelector::Private::loadHierarchy()
     (*it_m)->setExpanded(true);
   }
 
-  m_parent->m_description->clear();
-  if (m_parent->m_groupList->currentItem()) {
-    m_parent->m_description->setText(m_templates[m_parent->m_groupList->currentItem()->data(0, IdRole).toString()].longDescription());
+  ui->m_description->clear();
+  if (ui->m_groupList->currentItem()) {
+    ui->m_description->setText(m_templates[ui->m_groupList->currentItem()->data(0, Qt::UserRole).toInt()].longDescription());
   }
 }
 
-QList<MyMoneyTemplate> KAccountTemplateSelector::Private::selectedTemplates() const
+QList<MyMoneyTemplate> selectedTemplates() const
 {
   QList<MyMoneyTemplate> list;
-  QTreeWidgetItemIterator it(m_parent->m_groupList, QTreeWidgetItemIterator::Selected);
+  QTreeWidgetItemIterator it(ui->m_groupList, QTreeWidgetItemIterator::Selected);
   QTreeWidgetItem* it_v;
   while ((it_v = *it) != 0) {
-    list << m_templates[it_v->data(0, IdRole).toString()];
+    list << m_templates[it_v->data(0, Qt::UserRole).toInt()];
     ++it;
   }
   return list;
@@ -145,13 +135,30 @@ QList<MyMoneyTemplate> KAccountTemplateSelector::Private::selectedTemplates() co
 #endif
 
 
+
+public:
+  Ui::KAccountTemplateSelector    *ui;
+  QMap<QString, QTreeWidgetItem*>  m_templateHierarchy;
+#ifndef KMM_DESIGNER
+  QMap<int, MyMoneyTemplate>       m_templates;
+  // a map of country name or country name (language name) -> localeId (lang_country) so be careful how you use it
+  QMap<QString, QString>           countries;
+  QString                          currentLocaleId;
+  QMap<QString, QString>::iterator it_m;
+  QStringList                      dirlist;
+  int                              id;
+#endif
+};
+
 KAccountTemplateSelector::KAccountTemplateSelector(QWidget* parent) :
-    KAccountTemplateSelectorDecl(parent),
-    d(new Private(this))
+  QWidget(parent),
+  d_ptr(new KAccountTemplateSelectorPrivate)
 {
-  m_accountList->header()->hide();
-  m_groupList->setSelectionMode(QAbstractItemView::ExtendedSelection);
-  connect(m_groupList, SIGNAL(itemSelectionChanged()), this, SLOT(slotLoadHierarchy()));
+  Q_D(KAccountTemplateSelector);
+  d->ui->setupUi(this);
+  d->ui->m_accountList->header()->hide();
+  d->ui->m_groupList->setSelectionMode(QAbstractItemView::ExtendedSelection);
+  connect(d->ui->m_groupList, &QTreeWidget::itemSelectionChanged, this, &KAccountTemplateSelector::slotLoadHierarchy);
 
   // kick off loading of account template data
   QTimer::singleShot(0, this, SLOT(slotLoadTemplateList()));
@@ -159,48 +166,51 @@ KAccountTemplateSelector::KAccountTemplateSelector(QWidget* parent) :
 
 KAccountTemplateSelector::~KAccountTemplateSelector()
 {
+  Q_D(KAccountTemplateSelector);
   delete d;
 }
 
 void KAccountTemplateSelector::slotLoadTemplateList()
 {
 #ifndef KMM_DESIGNER
+  Q_D(KAccountTemplateSelector);
   QStringList dirs;
   // get list of template subdirs and scan them for the list of subdirs
-  d->dirlist = KGlobal::dirs()->findDirs("appdata", "templates");
+  d->dirlist = QStandardPaths::locateAll(QStandardPaths::DataLocation, "templates", QStandardPaths::LocateDirectory);
   QStringList::iterator it;
   for (it = d->dirlist.begin(); it != d->dirlist.end(); ++it) {
     QDir dir(*it);
-    // qDebug("Reading dir '%s' with %d entries", (*it).data(), dir.count());
     dirs = dir.entryList(QStringList("*"), QDir::Dirs);
     QStringList::iterator it_d;
     for (it_d = dirs.begin(); it_d != dirs.end(); ++it_d) {
       // we don't care about . and ..
       if ((*it_d) == ".." || (*it_d) == "." || (*it_d) == "C")
         continue;
-      QRegExp exp("(..)_(..)");
-      if (exp.indexIn(*it_d) != -1) {
-
-        QString country = KGlobal::locale()->countryCodeToName(exp.cap(2));
-        if (country.isEmpty())
-          country = exp.cap(2);
-        QString lang = KGlobal::locale()->languageCodeToName(exp.cap(1));
+      QLocale templateLocale(*it_d);
+      if (templateLocale.language() != QLocale::C) {
+        QString country = QLocale().countryToString(templateLocale.country());
+        QString lang = QLocale().languageToString(templateLocale.language());
         if (d->countries.contains(country)) {
           if (d->countries[country] != *it_d) {
-            QString oName = d->countries[country];
-            exp.indexIn(oName);
-            QString oCountry = KGlobal::locale()->countryCodeToName(exp.cap(2));
-            QString oLang = KGlobal::locale()->languageCodeToName(exp.cap(1));
+            QString otherName = d->countries[country];
+            QLocale otherTemplateLocale(otherName);
+            QString otherCountry = QLocale().countryToString(otherTemplateLocale.country());
+            QString otherLang = QLocale().languageToString(otherTemplateLocale.language());
             d->countries.remove(country);
-            d->countries[QString("%1 (%2)").arg(oCountry).arg(oLang)] = oName;
-            d->countries[QString("%1 (%2)").arg(country).arg(lang)] = *it_d;
+            d->countries[QString("%1 (%2)").arg(otherCountry, otherLang)] = otherName;
+            d->countries[QString("%1 (%2)").arg(country, lang)] = *it_d;
+            // retain the item corresponding to the current locale
+            if (QLocale().country() == templateLocale.country()) {
+              d->currentLocaleId = *it_d;
+            }
           }
         } else {
           d->countries[country] = *it_d;
+            // retain the item corresponding to the current locale
+          if (QLocale().country() == templateLocale.country()) {
+              d->currentLocaleId = *it_d;
+          }
         }
-      } else if ((*it_d).length() == 2) {
-        QString country = KGlobal::locale()->countryCodeToName((*it_d).toUpper());
-        d->countries[country] = *it_d;
       } else {
         qDebug("'%s/%s' not scanned", qPrintable(*it), qPrintable(*it_d));
       }
@@ -209,7 +219,7 @@ void KAccountTemplateSelector::slotLoadTemplateList()
 
   // now that we know, what we can get at max, we scan everything
   // and parse the templates into memory
-  m_groupList->clear();
+  d->ui->m_groupList->clear();
   d->m_templates.clear();
   d->it_m = d->countries.begin();
   d->id = 1;
@@ -224,27 +234,31 @@ void KAccountTemplateSelector::slotLoadTemplateList()
 void KAccountTemplateSelector::slotLoadCountry()
 {
 #ifndef KMM_DESIGNER
-  QTreeWidgetItem *parent = new QTreeWidgetItem(m_groupList);
+  Q_D(KAccountTemplateSelector);
+  QTreeWidgetItem *parent = new QTreeWidgetItem(d->ui->m_groupList);
   parent->setText(0, d->it_m.key());
   parent->setFlags(parent->flags() & ~Qt::ItemIsSelectable);
-  QStringList::iterator it;
-  for (it = d->dirlist.begin(); it != d->dirlist.end(); ++it) {
-    QStringList::iterator it_f;
-    QDir dir(QString("%1%2").arg(*it).arg(*(d->it_m)));
+  for (QStringList::iterator it = d->dirlist.begin(); it != d->dirlist.end(); ++it) {
+    QDir dir(QString("%1/%2").arg(*it).arg(*(d->it_m)));
     if (dir.exists()) {
       QStringList files = dir.entryList(QStringList("*"), QDir::Files);
-      for (it_f = files.begin(); it_f != files.end(); ++it_f) {
-        MyMoneyTemplate templ(QString("%1/%2").arg(dir.canonicalPath()).arg(*it_f));
-        d->m_templates[QString("%1").arg(d->id)] = templ;
+      for (QStringList::iterator it_f = files.begin(); it_f != files.end(); ++it_f) {
+        MyMoneyTemplate templ(QUrl::fromUserInput(QString("%1/%2").arg(dir.canonicalPath(), *it_f)));
+        d->m_templates[d->id] = templ;
         QTreeWidgetItem *item = new QTreeWidgetItem(parent);
         item->setText(0, templ.title());
         item->setText(1, templ.shortDescription());
-        item->setData(0, IdRole, QString("%1").arg(d->id));
+        item->setData(0, Qt::UserRole, QString("%1").arg(d->id));
         ++d->id;
       }
     }
   }
-
+  // make visible the templates of the current locale
+  if (d->it_m.value() == d->currentLocaleId) {
+    d->ui->m_groupList->setCurrentItem(parent);
+    d->ui->m_groupList->expandItem(parent);
+    d->ui->m_groupList->scrollToItem(parent, QTreeView::PositionAtTop);
+  }
   ++d->it_m;
   if (d->it_m != d->countries.end())
     QTimer::singleShot(0, this, SLOT(slotLoadCountry()));
@@ -258,6 +272,7 @@ void KAccountTemplateSelector::slotLoadCountry()
 void KAccountTemplateSelector::slotLoadHierarchy()
 {
 #ifndef KMM_DESIGNER
+  Q_D(KAccountTemplateSelector);
   d->loadHierarchy();
 #endif
 }
@@ -265,6 +280,7 @@ void KAccountTemplateSelector::slotLoadHierarchy()
 QList<MyMoneyTemplate> KAccountTemplateSelector::selectedTemplates() const
 {
 #ifndef KMM_DESIGNER
+  Q_D(const KAccountTemplateSelector);
   return d->selectedTemplates();
 #else
   return QList<MyMoneyTemplate>();
